/*
 BookViewContainerView.h
 BookShelf
 
 Created by Xinrong Guo on 12-2-24.
 Copyright (c) 2012 FoOTOo. All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 Redistributions of source code must retain the above copyright notice,
 this list of conditions and the following disclaimer.
 
 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 
 Neither the name of the project's author nor the names of its
 contributors may be used to endorse or promote products derived from
 this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
 TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <UIKit/UIKit.h>


@class GSBookShelfView;

@protocol GSBookShelfViewDelegate;
@protocol GSBookShelfViewDataSource;

typedef struct BookViewPostion {
    NSInteger row;
    NSInteger col;
    NSInteger index;
}BookViewPostion;

@interface GSBookViewContainerView : UIView {
    GSBookShelfView __unsafe_unretained *_parentBookShelfView;    
    
    @private
    
    CGRect _visibleRect;
    CGRect _availableRect;
    
    NSInteger _firstVisibleRow;
    NSInteger _lastVisibleRow;
    
    //NSInteger _firstVisibleIndex;
    //NSInteger _lastVisibleIndex;
    
    NSMutableArray *_visibleBookViews;
    NSMutableDictionary *_reuseableBookViews;
    
    CGFloat _bookViewWidth;
    CGFloat _bookViewHeight;
    CGFloat _bookViewSpacingWidth;
    
    // Drag and Drop
    BOOL _isDragViewPickedUp;
    UIView *_dragView;
    BookViewPostion _pickUpPosition;
    CGRect _pickUpRect;
    BOOL _isDragViewRemovedFromVisibleBookViews;
    
    BOOL _isBooksMoving;
    
    // Scroll While Drag
    CADisplayLink *_displayLink;
    double _lastDragScrollTime;
    
    // Remove Book
    NSMutableIndexSet *_indexsOfBookViewToBeRemoved;
    NSMutableIndexSet *_indexsOfBookViewNotShown;
    NSMutableArray *_tempVisibleBookViewCollector;
    BOOL _isRemoving;
    
    
}

@property (nonatomic, unsafe_unretained) GSBookShelfView *parentBookShelfView;

- (void)reloadData;

- (NSArray *)visibleBookViews;
- (UIView *)bookViewAtIndex:(NSInteger)index;

- (void)layoutSubviewsWithAvailableRect:(CGRect)availableRect visibleRect:(CGRect)visibleRect;

- (UIView *)dequeueReusableBookViewWithIdentifier:(NSString *)identifier;

- (void)removeBookViewsAtIndexs:(NSIndexSet *)indexs animate:(BOOL)animate;
- (void)insertBookViewsAtIndexs:(NSIndexSet *)indexs animate:(BOOL)animate;

@end
